/**
 *  @author     TTF006
 *  @version    $Id: $
 *  @desc       This module provides dummuy connection - for unsupported ports for VxLTE.
 *  @see        
 */
package com.iskratel.ttcn.tci.codec.IOTCodec;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.lang.reflect.Method;

import org.etsi.ttcn.tci.BooleanValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.OctetstringValue;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.RecordOfValue;
import org.etsi.ttcn.tci.TciCDProvided;
 import org.etsi.ttcn.tci.TciCDRequired;
 import org.etsi.ttcn.tci.TciTMProvided;
import org.etsi.ttcn.tci.TciTypeClass;
import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.UnionValue;
import org.etsi.ttcn.tci.UniversalCharstringValue;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tri.TriComponentId;
import org.etsi.ttcn.tri.TriMessage;

import com.iskratel.protocol.diameter.parser.Codec;
import com.iskratel.protocol.diameter.parser.DiameterMessageDecodeException;
import com.testingtech.ttcn.extension.CodecProvider;
import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tri.AbstractCodecPlugin;

import com.testingtech.ttcn.logging.TciTLProvidedV321TT;
import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.BitString;
import de.tu_berlin.cs.uebb.ttcn.runtime.ForeignMessage;

public class IOTCodec extends AbstractBaseCodec implements TciCDProvided {

	private static final int CAPTURED_PACKET_IND  =   0x0001; /**< ID of CaptureData */
	private static final int SET_FILTER_REQ       =   0x0002; /**< ID of SetFilterRequest */
	private static final int START_CAPTURE_REQ    =   0x0003; /**< ID of StartCaptureRequest */
	private static final int START_CAPTURE_REP    =   0x0004; /**< ID of StartCaptureReply */
	private static final int STOP_CAPTURE_REQ     =   0x0005; /**< ID of StopCaptureRequest */
	private static final int STOP_CAPTURE_REP     =   0x0006; /**< ID of StopCaptureReply */
	private static final int OPEN_DEVICE_REQ	  =	  0x0007; /**< ID of OpenDeviceRequest */
	private static final int OPEN_DEVICE_REP      =   0x0008; /**< ID of OpenDeviceReply */
	private static final int SET_FILTER_REP       =   0x0009; /**< ID of SetFilterReply */
	private static final int MERGE_FILES_REQ      =   0x000A; /**< ID of MergePcapFilesRequest */
	private static final int MERGE_FILES_REP      =   0x000B; /**< ID of MergePcapFilesReply */

	protected TciTLProvidedV321TT logging;
	
	protected TciCDRequired typeServer;
	protected TciTMProvided management;
	
	private Type booleanType;
	private Type octetstringType;
	private Type charstringType;
	private Type universalcharstringType;
	private Type integerType;
	private Type bigintegerType;
		
	private boolean padding = false;
	
	/**<!--###################################################################-->
	 *
	  Constructor method <b>DefaultCodec</b> creates new object and prepares RB.
	 * <p>
	 * @param rb needed to resolve specific information (e.g. which logging to 
	 *           use)
	 *         
	 * <!--################################################################--> */  
	public IOTCodec(RB rb) {
	  super(rb);
	  //super();
      typeServer = /*getRB().getTciCDRequired();*/RB.getTciCDRequired();
	  octetstringType = typeServer.getOctetstring();
	  charstringType = typeServer.getCharstring();
	  universalcharstringType = typeServer.getUniversalCharstring();
	  integerType = typeServer.getInteger();
	  bigintegerType = typeServer.getFloat();
	  booleanType = typeServer.getBoolean();
	  management = /*getRB().getTciTMProvided();*/RB.getTciTMProvided();	
	}
	
	@Override
	public Value decode(TriMessage message, Type decodingHypothesis) {
		//Decoding aligned to old TA LibIot	
		// Get the received byte array 
		encodedMsg = message.getEncodedMessage();
	    // Set the bitpos to 0 (this is needed to know the actual bit position from the byte array)
	    bitpos = 0;
	    // The the encoded message is null, nothing can be done, so return null 
	    if(encodedMsg.length == 0) return null;
	
        // Check which type class the decoding hypothesis is of
        switch (decodingHypothesis.getTypeClass()) {
            case TciTypeClass.BOOLEAN:
	    	  	BooleanValue booleanValue = (BooleanValue) booleanType.newInstance();
				
	    	  	if (encodedMsg[0] == 0x00) {booleanValue.setBoolean(false);}
				else {booleanValue.setBoolean(true);}
				
	    	  	return booleanValue;
	        case TciTypeClass.RECORD:
	        	
	        	int iotmsgType=encodedMsg[0]&0xFF | encodedMsg[1]&0xFF << 8; 
	        	int iotmsgLen=2+4+(((encodedMsg[2] & 0xFF))|((encodedMsg[3] & 0xFF) << 8)|((encodedMsg[4] & 0xFF) << 16) |  ((encodedMsg[5] & 0xFF)<<24));
	           
	        	switch (iotmsgType){
	            case CAPTURED_PACKET_IND:
	            case SET_FILTER_REQ:
	            case START_CAPTURE_REQ:
	            case STOP_CAPTURE_REQ:
	            case OPEN_DEVICE_REQ:
	            case MERGE_FILES_REQ:
	            default:
	            	//*_REP
	            	// Fill the NewHypotesis
		            /*type record GeneralConfigurationRsp
		            {
		              Status status
		            }
		            
		            type record SetFilterRsp
		            {
		              Status status
		            }
		            
		            type record StartTrafficCaptureRsp
				    {
				      Status status
				    }
				    
				    type record StopTrafficCaptureRsp
				    {
				      Status status
				    }
		            */
	            	int iotmsgStatus = encodedMsg[2+4]&0xFF;//0+6=7thposition
		            RecordValue rv = (RecordValue) decodingHypothesis.newInstance();
		            
		            /*type record Status {
		      			FncRetCode code,
		      			charstring reason optional
		      		}*/
		            RecordValue rvstatus = (RecordValue) rv.getField("status").getType().newInstance();
		            
		            /*type enumerated FncRetCode {
		      		  e_success(0),
		      		  // error codes
		      		  e_error(1),
		      		  e_timeout(2)
		      		}*/
		            EnumeratedValue code = (EnumeratedValue) rvstatus.getField("code").getType().newInstance();
		            //code.setEnum("e_success");
		            code.setInt(iotmsgStatus);
		            rvstatus.setField("code",code);
		            
		            if (iotmsgLen>2+4+1){
		                CharstringValue r = (CharstringValue) rvstatus.getField("reason").getType().newInstance();
		                r.setString("OK");
		                rvstatus.setField("reason",r);
		            }
		            //return createNewHypotesis(NewHypotesis,encodedMsg);
		            //return Newhypotesis;
		            rv.setField("status", rvstatus);
		            return rv;
	            }
	        	
            default:
                return super.decode(message,decodingHypothesis);
        }  
		//return null;
	}

	@Override
	public TriMessage encode(Value value) {
		// The template is given to the AbstractBaseCodec which handles the basic encoding
	    //TriMessage encodedMessage = super.encode(template);
	    //return encodedMessage;
		
		Type msgType = value.getType();
		final ByteArrayOutputStream out = new ByteArrayOutputStream();
			 
		switch (msgType.getTypeClass()) {
				
			case TciTypeClass.INTEGER:
			case TciTypeClass.FLOAT:
			case TciTypeClass.BITSTRING:
			case TciTypeClass.HEXSTRING:
			case TciTypeClass.OCTETSTRING:
			case TciTypeClass.CHARSTRING:
			case TciTypeClass.UNIVERSAL_CHARSTRING:
			case TciTypeClass.RECORD_OF:
			case TciTypeClass.ENUMERATED:
			case TciTypeClass.UNION:
			case TciTypeClass.BOOLEAN:
			case TciTypeClass.VERDICT:
			case TciTypeClass.RECORD:
							  return encode_IOTMessage(out, (RecordValue) value);
			default:
				//throw new TciException("Unknown TypeClass");
			    logging.tliLog("", System.currentTimeMillis(), "", -1, null, "IOT codec: encode: Unknown TypeClass" + msgType + msgType.getTypeClass());
				    
		}
		return null;
	}
	
	 public TriMessage encode_IOTMessage(ByteArrayOutputStream out, RecordValue value_) {
		 	//Encoding aligned to old TA LibIot		    
		    if (value_.getType().getName().equals("GeneralConfigurationReq")){ 
		    	
		    	/*charstring  captureProcessIpAddress,
		        PortNumber  captureProcessPort,
		        CaptureMode captureMode,
		        EutInterfaceInfoList eutInfoList optional*/
		    	
//		    	encode_Charstring(out, (CharstringValue)value_.getField("captureProcessIpAddress"));
//		    	encode_IntegerValue(out,(IntegerValue) value_.getField("captureProcessPort"));
//		    	encode_CaptureMode(out,(UnionValue) value_.getField("captureMode"));
//		    	encode_EutInterfaceInfoList(out,(RecordOfValue) value_.getField("eutInfoList"));
		    	//Implementation due to existing CapturePCO
		    	//Type
		    	encode_UChar(out,0);
		    	//Mode
		    	UnionValue value=(UnionValue)value_.getField("captureMode");
		    	if (value.getPresentVariantName().equals("liveCpature") && !value.getVariant("liveCpature").notPresent()) {
		    		encode_UChar(out,0);
				} 
		    	else if (value.getPresentVariantName().equals("offlineCapture") && !value.getVariant("offlineCapture").notPresent()) {
		    		encode_UChar(out,1);
				} 
		    	
		    	//Timestamp
		    	encode_Time(out,0,0);
		    	//params
		    	encode_StringWithLen(out, (CharstringValue)((RecordValue)((RecordValue)value.getVariant("offlineCapture")).getField("captureSource")).getField("sessionsPath"));
		    	//capturefile
		    	CharstringValue file =(CharstringValue) charstringType.newInstance();
		    	file.setString("");
		    	encode_StringWithLen(out, file);
		    	
		    	encode_MsgTypeWithLen(out, OPEN_DEVICE_REQ);
		    }
		    //else if(value_.getType().getName().equals("GeneralConfigurationRsp")){}
		    else if (value_.getType().getName().equals("SetFilterReq")){ 
		    	
		    	//tcpdump -qns 0 -XX  \(\(tcp or sctp\) and host 172.29.13.100 and port 65535 and \(host 192.168.50.81 or 192.168.50.82\) and port 3868\) -r /home/ttcn3/workspace/TTF006_VxLTE/tracefiles/TC_VxLTE_INT_REG_01.pcapng

		    	ByteArrayOutputStream filter = new ByteArrayOutputStream();
		    	write(filter,"(");
		       /*type record SetFilterReq {
		          ProtocolFilter    protocol,
		          InterfaceInfoList interfaceInfos
		        }*/
		    	encode_ProtocolFilter(filter, (EnumeratedValue)value_.getField("protocol"));
		    	
		    	write(filter," and ");
		    	encode_InterfaceInfoList(filter,(RecordOfValue) value_.getField("interfaceInfos"));
		    	
		    	write(filter,")");
		    	
		    	//Implementation due to existing CapturePCO
		    	CharstringValue cvfilter =(CharstringValue) charstringType.newInstance();
		    	cvfilter.setString(filter.toString());// port 5080");
		    	//filter.setString("tcp port 5501");
		    	encode_StringWithLen(out, cvfilter);
		    	
		    	encode_MsgTypeWithLen(out, SET_FILTER_REQ);
		    }
		    //else if (value_.getType().getName().equals("SetFilterRsp")){}
		    else if (value_.getType().getName().equals("StartTrafficCaptureReq")){ 
		    	
		    	encode_MsgTypeWithLen(out, START_CAPTURE_REQ);
		    }
		    //else if (value_.getType().getName().equals("StartTrafficCaptureRsp")){}
		    else if (value_.getType().getName().equals("StopTrafficCaptureReq")){ 
		    	
		    	encode_MsgTypeWithLen(out, STOP_CAPTURE_REQ);
		    }
		    //else if (value_.getType().getName().equals("StopTrafficCaptureRsp")){}	    
		    else{
		    	/*error*/
		    }
			return new ForeignMessage(/*this.getRB(), */out.toByteArray());
	}
	
	private void encode_MsgTypeWithLen(ByteArrayOutputStream out, int MsgType){
		final ByteArrayOutputStream mt_len = new ByteArrayOutputStream();
		write(mt_len,new byte[]{(byte)(MsgType),(byte)(MsgType>>>8)});//little endian
		//encode_IntegerValue(mt_len, (/*2+4+*/out.size()));
		write(mt_len,new byte[]{(byte)(out.size()),(byte)(out.size() >>> 8), (byte)(out.size()>>> 16), (byte)(out.size()>>> 24)});//little endian
		 try {
			 out.writeTo(mt_len);
			 out.reset();
			 mt_len.writeTo(out);
		 }
		 catch(Exception e){e.printStackTrace();}
		
	} 
	
	////GeneralConfigurationReq params
	private void encode_PhysicalInterfaceList(ByteArrayOutputStream out,	RecordOfValue value){
		/*type record of charstring PhysicalInterfaceList;*/
		if ((value == null) || value.notPresent()) {return;}
		   
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			//RecordValue rv=(RecordValue) value.getField(i);
			encode_Charstring(out, (CharstringValue)value.getField(i));	
		}		
		return;
	}
	 
    private void encode_RecordMode(ByteArrayOutputStream out,	EnumeratedValue value){
    	/* type enumerated RecordMode {
   	      e_norecord,
   	      e_record  
   	    }*/
	    encode_IntegerValue(out, value.getInt());
	}
    
	private void encode_LiveCapture(ByteArrayOutputStream out,	RecordValue value){
		/*type record LiveCapture {
    	PhysicalInterfaceList physicalInterfaces, // TODO document example syntax
         RecordMode            recordMode
        }*/
		encode_PhysicalInterfaceList(out,(RecordOfValue)value.getField("physicalInterfaces"));
		encode_RecordMode(out,(EnumeratedValue)value.getField("recordMode") );		 
	}
	
	
	private void encode_TimeOffset(ByteArrayOutputStream out,	RecordValue value){
		/*type record TimeOffset
	    {
	      //UInt32 seconds,
	      //UInt32 microseconds
	      integer seconds (0..4294967295),
		  integer microseconds (0..4294967295)
	    }*/
    	encode_IntegerValue(out,(IntegerValue) value.getField("seconds"));
    	encode_IntegerValue(out,(IntegerValue) value.getField("microseconds"));
	}
	
	
	private void encode_FileList(ByteArrayOutputStream out,	RecordOfValue value){
		/*type record of charstring FileList;*/
		if ((value == null) || value.notPresent()) {return;}
	    
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			//RecordValue rv=(RecordValue) value.getField(i);
			encode_Charstring(out, (CharstringValue)value.getField(i));	
		}		
		return;
	} 

	private void encode_MergeFileList(ByteArrayOutputStream out,	RecordValue value){
		 /*type record MergeFileList {
		      FileList    mergeFileList, 
		      charstring  mergeFilePath,   
		      charstring  mergeToolPath   
		  }*/
		encode_FileList(out, (RecordOfValue)value.getField("mergeFileList"));
		encode_Charstring(out, (CharstringValue)value.getField("mergeFilePath"));
		encode_Charstring(out, (CharstringValue)value.getField("mergeToolPath"));	
	} 
   
	
	private void encode_CaptureSource(ByteArrayOutputStream out,	RecordValue value){
	    /*type record CaptureSource {
		  charstring sessionsPath, // e.g., the path where session's PCAP files are located like /tmp/19
		  MergeFileList mergeFileList
		 }*/
		encode_Charstring(out, (CharstringValue)value.getField("sessionsPath"));
		encode_MergeFileList(out, (RecordValue)value.getField("mergeFileList"));
	}
	
	private void encode_OfflineCapture(ByteArrayOutputStream out,	RecordValue value){
		   /* type record OfflineCapture {
		      TimeOffset     offset,
		      CaptureSource  captureSource
		    }*/
		    encode_TimeOffset(out,(RecordValue)value.getField("offset"));
		    encode_CaptureSource(out,(RecordValue)value.getField("captureSource"));
	}		 
	 
	private void encode_CaptureMode(ByteArrayOutputStream out,	UnionValue value){
		/*type union CaptureMode {
		    LiveCapture liveCpature,
		    OfflineCapture offlineCapture
		}*/
		if (value.getPresentVariantName().equals("liveCpature") && !value.getVariant("liveCpature").notPresent()) {
			encode_LiveCapture(out, (RecordValue) value.getVariant("liveCpature"));
		} 
		if (value.getPresentVariantName().equals("offlineCapture") && !value.getVariant("offlineCapture").notPresent()) {
			encode_OfflineCapture(out, (RecordValue) value.getVariant("offlineCapture"));
		} 
	}
	
	private void encode_EutInterfaceInfo(ByteArrayOutputStream out,	RecordValue value){
	    /*type record EutInterfaceInfo {
		     charstring     eut,
		     IpAddress      ipAddress,
		     PortNumber     portNumber
		}*/
		encode_Charstring(out, (CharstringValue)value.getField("eut"));
		encode_Charstring(out, (CharstringValue)value.getField("ipAddress"));
    	encode_IntegerValue(out,(IntegerValue) value.getField("portNumber"));
	}
	
	private void encode_EutInterfaceInfoList(ByteArrayOutputStream out,	RecordOfValue value){
		if ((value == null) || value.notPresent()) {return;}
	    
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			//RecordValue rv=(RecordValue) value.getField(i);
			encode_EutInterfaceInfo(out, (RecordValue)value.getField(i));	
		}		
		return;
	} 
	 
	////set-filter-REQ params
  	
	private void encode_ProtocolFilter(ByteArrayOutputStream out,	EnumeratedValue value){
		/*type enumerated ProtocolFilter {
	        e_ip,
	    	e_sip,
	    	e_dns,
	    	e_diameter
	    }*/
		String protocol="ip"; //default
		switch (value.getInt()){
		
		case 0:protocol= "ip";break;
		case 1:protocol= "udp";break;
		case 2:/**/
		case 3:protocol= "(tcp or sctp)";break;
		default:
		
		}
		protocol="ip"; //default
    	//encode_IntegerValue(out, value.getInt());
    	write(out, protocol);
	}
	
	private void encode_PortNumberList(ByteArrayOutputStream out,	RecordOfValue value){
		if ((value == null) || value.notPresent()) {return;}
	    
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			//RecordValue rv=(RecordValue) value.getField(i);
			//encode_IntegerValue(out,(IntegerValue)value.getField(i));
			//encode_Charstring(out,(CharstringValue)value.getField(i));
			write(out,"port ");
			write(out,value.getField(i).toString());
		}		
		return;
	} 
	
	private void encode_IPInterfaceInfo(ByteArrayOutputStream out,	RecordValue value){
		/*type record IpInterfaceInfo {
		   charstring domainName optional,
		   IpAddress IpAddress,
		   PortNumberList portNumbers	
		}*/
		//encode_Charstring(out, (CharstringValue)value.getField("domainName"));
		write(out,"host ");
		encode_Charstring(out, (CharstringValue)value.getField("IpAddress"));
		//write(out," and ");//commented out because of IP fragmentation
		//encode_PortNumberList(out,(RecordOfValue)value.getField("portNumbers"));//commented out because of IP fragmentation
		
	}
	
	private void encode_IpInterfaceInfoList(ByteArrayOutputStream out,	RecordOfValue value){
		/*type record of IpInterfaceInfo IpInterfaceInfoList;*/
		if ((value == null) || value.notPresent()) {return;}
	    
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			//RecordValue rv=(RecordValue) value.getField(i);
			//write(out,"(");
			encode_IPInterfaceInfo(out, (RecordValue)value.getField(i));
			if (i < (len-1)) write(out," and ");
			//write(out,")");
		}		
		return;
	} 
	
	private void encode_InterfaceInfo(ByteArrayOutputStream out,	UnionValue value){
	    /*type union InterfaceInfo {
		   IpInterfaceInfoList IpInterfaceInfo
		}*/
		if (value.getPresentVariantName().equals("IpInterfaceInfo") && !value.getVariant("IpInterfaceInfo").notPresent()) {
			encode_IpInterfaceInfoList(out, (RecordOfValue) value.getVariant("IpInterfaceInfo"));
		} 		
	}
	
	private void encode_InterfaceInfoList(ByteArrayOutputStream out,	RecordOfValue value){
		/*type record of InterfaceInfo InterfaceInfoList;*/
		if ((value == null) || value.notPresent()) {return;}
	    
		int len = value.getLength();
		for (int i = 0; i < len; i++) {
			write(out,"(");
			//RecordValue rv=(RecordValue) value.getField(i);
			encode_InterfaceInfo(out, (UnionValue)value.getField(i));	
			write(out,")");
		}		
		return;
	} 
	 
	    private void encode_UChar(ByteArrayOutputStream out, int value) {
		
		    //if ((value == null) || value.notPresent()) {return;}

		    int ivalue=value;
			byte[] IntVal= new byte[]{(byte)ivalue};
			write(out,IntVal);
		
	    }
	    
	    private void encode_StringWithLen(ByteArrayOutputStream out, CharstringValue value) {
			
			if ((value == null) || value.notPresent()) {return;}

			StringBuffer sb = new StringBuffer();
            int len = value.getLength();
			for (int i = 0; i < len; i++) {
				sb.append(value.getChar(i));
			}
			write(out,new byte[]{(byte)(len),(byte)(len >>> 8), (byte)(len>>> 16), (byte)(len>>> 24)});//little endian
			
			write(out, sb.toString());
			
			//if (padding)
			 //   write_avp_padding(out,value.getLength());
		}
	 
		private void encode_Charstring(ByteArrayOutputStream out, CharstringValue value) {
			
			if ((value == null) || value.notPresent()) {return;}

			StringBuffer sb = new StringBuffer();

			for (int i = 0; i < value.getLength(); i++) {
				sb.append(value.getChar(i));
			}
			write(out, sb.toString());
			
			if (padding)
			    write_avp_padding(out,value.getLength());
		}
		
		private void encode_UniversalCharstring(ByteArrayOutputStream out, UniversalCharstringValue value) {
			
			if ((value == null) || value.notPresent()) {return;}
	     
			StringBuffer sb = new StringBuffer();
	        sb.append(value.getString());
			write(out, sb.toString());
			if (padding)
			    write_avp_padding(out,sb.length());
		}
		
		private void encode_Octetstring(ByteArrayOutputStream out, OctetstringValue value) {
			
			if ((value == null) || value.notPresent()) {return;}

			byte[] OctetVal= new byte[value.getLength()];

			for (int i = 0; i < value.getLength(); i++) {
				OctetVal[i]=(byte)value.getOctet(i);
			}
			write(out, OctetVal);
			if (padding)
			    write_avp_padding(out,value.getLength());
		}
		
		private void encode_IntegerValue(ByteArrayOutputStream out, IntegerValue value) {
			
			if ((value == null) || value.notPresent()) {return;}
			
			int ivalue=value.getInt(); 
			byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
	                                  (byte)(ivalue >>> 16),
	                                  (byte)(ivalue >>> 8),
	                                  (byte)ivalue};	
			write(out,IntVal);
		}
		
		private void encode_IntegerValue(ByteArrayOutputStream out, int value) {
			
			int ivalue=value;
			byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
	                                  (byte)(ivalue >>> 16),
	                                  (byte)(ivalue >>> 8),
	                                  (byte)ivalue};
			write(out,IntVal);
		}
		
		private void encode_Integer64Value(ByteArrayOutputStream out, IntegerValue value) {
			
			if ((value == null) || value.notPresent()) {return;}
			
			//byte[] IntVal= new byte[8];
			//IntVal = String.valueOf(value).getBytes();
			
			int ivalue=value.getInt();
			byte[] IntVal64= new byte[]{
					(byte)((ivalue& 0xffffffffL) >>> 56),
					(byte)((ivalue& 0xffffffffL) >>> 48),
	                (byte)((ivalue& 0xffffffffL) >>> 40),
	                (byte)((ivalue& 0xffffffffL) >>> 32),
					(byte)((ivalue& 0xffffffffL) >>> 24),
	                (byte)((ivalue& 0xffffffffL) >>> 16),
	                (byte)((ivalue& 0xffffffffL) >>> 8),
	                (byte)(ivalue& 0xffffffffL)};
			
			write(out,IntVal64);
			
		}
		
		private void encode_Integer64Value(ByteArrayOutputStream out, long ivalue) {
			
			//byte[] IntVal= new byte[8];
			//IntVal = String.valueOf(ivalue).getBytes();
			
			byte[] IntVal64= new byte[]{
					(byte)((ivalue& 0xffffffffL) >>> 56),
					(byte)((ivalue& 0xffffffffL) >>> 48),
	                (byte)((ivalue& 0xffffffffL) >>> 40),
	                (byte)((ivalue& 0xffffffffL) >>> 32),
					(byte)((ivalue& 0xffffffffL) >>> 24),
	                (byte)((ivalue& 0xffffffffL) >>> 16),
	                (byte)((ivalue& 0xffffffffL) >>> 8),
	                (byte)(ivalue& 0xffffffffL)};
			
			write(out,IntVal64);
			//System.out.println(ivalue+" "+IntVal64);
		}
        private void encode_Time(ByteArrayOutputStream out, int values,int valuems) {
			
			int ivalue=values;
			byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
	                                  (byte)(ivalue >>> 16),
	                                  (byte)(ivalue >>> 8),
	                                  (byte)ivalue};
			write(out,IntVal);
			ivalue=valuems;
			IntVal= new byte[]{(byte)(ivalue >>> 24),
	                                  (byte)(ivalue >>> 16),
	                                  (byte)(ivalue >>> 8),
	                                  (byte)ivalue};
			write(out,IntVal);
			ivalue=0;//valueus;
			IntVal= new byte[]{(byte)(ivalue >>> 24),
	                                  (byte)(ivalue >>> 16),
	                                  (byte)(ivalue >>> 8),
	                                  (byte)ivalue};
			write(out,IntVal);
		}

		private void write_avp_padding(ByteArrayOutputStream out, int avp_data_len) {

			final byte[] NullVal = {0x00};
			
			if ((avp_data_len % 4)!=0){
				for (int i = 0; i < 4-((avp_data_len % 4)); i++) {	
					write(out, NullVal);
				}
			}
		}
		
		private void write(ByteArrayOutputStream out, String outString) {
			
			try {
				out.write(outString.getBytes());
			} catch (IOException iex) {
				return;
			}
			return;
		}
		
		private void write(ByteArrayOutputStream out, byte [] outString) {
			
			try {
				out.write(outString);
			} catch (IOException iex) {
				return;
			}
			return;
		}	


}
