/**
 *  @author     TTF006
 *  @version    $Id: $
 *  @desc       This module provides dummuy connection - for unsupported ports for VxLTE.
 *  @see        
 */
package com.iskratel.ttcn.tci.codec.LibUpperTesterCodec;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.etsi.ttcn.tci.BooleanValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.OctetstringValue;
import org.etsi.ttcn.tci.RecordOfValue;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.TciCDProvided;
 import org.etsi.ttcn.tci.TciCDRequired;
 import org.etsi.ttcn.tci.TciTMProvided;
import org.etsi.ttcn.tci.TciTypeClass;
import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.UnionValue;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tri.TriMessage;

import com.testingtech.ttcn.extension.CodecProvider;
import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tri.AbstractCodecPlugin;

import com.testingtech.ttcn.logging.TciTLProvidedV321TT;
import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.BitString;
import de.tu_berlin.cs.uebb.ttcn.runtime.ForeignMessage;
  
public class LibUpperTesterCodec extends AbstractBaseCodec implements TciCDProvided {

	protected TciTLProvidedV321TT logging;
	protected TciCDRequired typeServer;
	protected TciTMProvided management;
	
	private Type booleanType;
	private Type octetstringType;
	private Type charstringType;
	private Type universalcharstringType;
	private Type integerType;
	private Type bigintegerType;
	
	@Override
	public Value decode(TriMessage message, Type decodingHypothesis) {
		// Get the received byte array 
		encodedMsg = message.getEncodedMessage();
        // Set the bitpos to 0 (this is needed to know the actual bit position from the byte array)
	    bitpos = 0;
	    // The the encoded message is null, nothing can be done, so return null 
	    if(encodedMsg.length == 0) return null;
	
        // Check which type class the decoding hypothesis is of
        switch (decodingHypothesis.getTypeClass()) {
            case TciTypeClass.BOOLEAN:
                BooleanValue booleanValue = (BooleanValue) booleanType.newInstance();
				
	    	  	if (encodedMsg[0] == 0x00) {booleanValue.setBoolean(false);}
				else {booleanValue.setBoolean(true);}
				
	    	  	return booleanValue;
            case TciTypeClass.RECORD:
                 
            	if (decodingHypothesis.getName().equals("EquipmentOperationReq")) {
            		/*type record EquipmentOperationReq {
            			EquipmentCommand cmd, ParameterList params optional 
            		}*/
            		RecordValue rv = (RecordValue) decodingHypothesis.newInstance();
            		
            		CharstringValue ccmd = (CharstringValue) rv.getField("cmd").getType().newInstance();
     	            ccmd.setString("COMMAND");  //parse command from message       
     	            rv.setField("cmd",ccmd);
     	            //decode_ParameterList()   //parse parameterlist from message
            		return rv;
            	}
            	else if (decodingHypothesis.getName().equals("EquipmentOperationRsp")){
		            /*type record EquipmentOperationRsp {
				      Status status
			        }*/
		            RecordValue rv = (RecordValue) decodingHypothesis.newInstance();
		                          
		            RecordValue rvstatus = (RecordValue) rv.getField("status").getType().newInstance();
		            
		      		/*type enumerated FncRetCode {
		      		  e_success(0),
		      		  // error codes
		      		  e_error(1),
		      		  e_timeout(2)
		      		}*/	 
		            EnumeratedValue code = (EnumeratedValue) rvstatus.getField("code").getType().newInstance();
		            code.setEnum("e_success");
		            
		            CharstringValue r = (CharstringValue) rvstatus.getField("reason").getType().newInstance();
		            r.setString("OK");
		            rvstatus.setField("code",code);
		            rvstatus.setField("reason",r);
		            
		            //return createNewHypotesis(NewHypotesis,encodedMsg);
		            //return Newhypotesis;
		            rv.setField("status", rvstatus);
		            return rv;
            	}

            default:
                return super.decode(message,decodingHypothesis);
        }  
		//return null;
	}

	@Override
	public TriMessage encode(Value value) {
		// The template is given to the AbstractBaseCodec which handles the basic encoding
        //TriMessage encodedMessage = super.encode(template);
        //return encodedMessage;
	    
		Type msgType = value.getType();
		final ByteArrayOutputStream out = new ByteArrayOutputStream();
		
		 switch (msgType.getTypeClass()) {
			
			case TciTypeClass.INTEGER:
			case TciTypeClass.FLOAT:
			case TciTypeClass.BITSTRING:
			case TciTypeClass.HEXSTRING:
			case TciTypeClass.OCTETSTRING:
			case TciTypeClass.CHARSTRING:
			case TciTypeClass.UNIVERSAL_CHARSTRING:
			case TciTypeClass.RECORD_OF:
			case TciTypeClass.ENUMERATED:
			case TciTypeClass.UNION:
			case TciTypeClass.BOOLEAN:
			case TciTypeClass.VERDICT:
			case TciTypeClass.RECORD:
			                  return encode_LibUpperTesterMessage(out, (RecordValue) value);
			default:
				//throw new TciException("Unknown TypeClass");
			    logging.tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, "LibUpperTester codec: encode: Unknown TypeClass" + msgType + msgType.getTypeClass());	
			}
		return null;
	}
	
	public TriMessage encode_LibUpperTesterMessage(ByteArrayOutputStream out, RecordValue value_) {
		  
		if (value_.getType().getName().endsWith("Req")) {
			encode_EquipmentOperationReq(out, value_);
		}

		if (value_.getType().getName().equals("Rsp")) {
			encode_EquipmentOperationRsp(out, value_);
		}	
		
		return new ForeignMessage(RB, out.toByteArray());
	}
	
	private void encode_ParameterList(ByteArrayOutputStream out, RecordOfValue rov){
		
		if ((rov == null) | rov.notPresent()) {return;}
	    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			encode_Charstring(out, (CharstringValue)rov.getField(i));
			write(out," ".getBytes());
		}
		return;
	}

	private void encode_EquipmentOperationReq(ByteArrayOutputStream out, RecordValue value_) {
		
		encode_Charstring(out,(CharstringValue)value_.getField("cmd"));//EquipmentCommand cmd, 
		write(out,"\n".getBytes());
		encode_ParameterList(out,(RecordOfValue)value_.getField("params"));// params optional 
		return;
	}
	
	private void encode_EquipmentOperationRsp(ByteArrayOutputStream out, RecordValue value_) {
		
		RecordValue rvstatus = (RecordValue)value_.getField("status");
		
		encode_IntegerValue(out,(IntegerValue)rvstatus.getField("code"));
		encode_Charstring(out,(CharstringValue)rvstatus.getField("reason"));	
		return;
	}
	 
	private void write(ByteArrayOutputStream out, byte [] outString) {
			
		try {
			out.write(outString);
		} catch (IOException iex) {
			return;
		}
		return;
	}
	
    private void encode_Charstring(ByteArrayOutputStream out, CharstringValue value) {
		
		if ((value == null) || value.notPresent()) {return;}

		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < value.getLength(); i++) {
			sb.append(value.getChar(i));
		}
		write(out, sb.toString().getBytes());
		return;
	}
    
	private void encode_IntegerValue(ByteArrayOutputStream out, IntegerValue value) {
		
		if ((value == null) || value.notPresent()) {return;}
		
		int ivalue=value.getInt(); 
		byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
                                  (byte)(ivalue >>> 16),
                                  (byte)(ivalue >>> 8),
                                  (byte)ivalue};	
		write(out,IntVal);
	}
}
