/**
 *  @author     TTF006
 *  @version    $Id: $
 *  @desc       This module provides test adapter for VxLTE.
 *  @see        
 */
package com.iskratel.ttcn.tri;


import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tci.TciCDProvided;
import org.etsi.ttcn.tci.TciCDRequired;
import org.etsi.ttcn.tci.TciModuleId;
import org.etsi.ttcn.tci.TciModuleParameterId;
import org.etsi.ttcn.tci.TciParameterList;
import org.etsi.ttcn.tci.TciTMProvided;
import org.etsi.ttcn.tri.TriAddress;
import org.etsi.ttcn.tri.TriCommunicationSA;
import org.etsi.ttcn.tri.TriComponentId;
import org.etsi.ttcn.tri.TriFunctionId;
import org.etsi.ttcn.tri.TriMessage;
import org.etsi.ttcn.tri.TriParameter;
import org.etsi.ttcn.tri.TriParameterList;
import org.etsi.ttcn.tri.TriPlatformPA;
import org.etsi.ttcn.tri.TriPortId;
import org.etsi.ttcn.tri.TriPortIdList;
import org.etsi.ttcn.tri.TriStatus;
import org.etsi.ttcn.tri.TriTestCaseId;

import com.iskratel.ttcn.tci.codec.DIAMETERCodec.DIAMETERCodec;
import com.iskratel.ttcn.tci.codec.IOTCodec.IOTCodec;
import com.iskratel.ttcn.tci.codec.LibUpperTesterCodec.LibUpperTesterCodec;
import com.iskratel.ttcn.tci.codec.SIPCodec.SIPCodec;
import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.logging.TciTLProvidedV321TT;
import com.testingtech.ttcn.tci.TciEncoding;
import com.testingtech.ttcn.tci.TestCaseParameterServer;
import com.testingtech.ttcn.tri.TestAdapter;
import com.testingtech.ttcn.tci.TciModuleParameterIdImpl;
//import com.testingtech.ttcn.tci.TciParameterImpl;
import com.testingtech.ttcn.tci.TciParameterListImpl;
//import com.testingtech.ttcn.tci.TciModuleParameterIdImpl;
import com.testingtech.ttcn.tri.TriStatusImpl;


import de.tu_berlin.cs.uebb.muttcn.runtime.RB;


public class VxlteAdapter extends TestAdapter implements TriCommunicationSA,	TriPlatformPA, TciEncoding, TestCaseParameterServer {

	private static final long serialVersionUID = 1L;

	static Logger log = Logger.getLogger(VxlteAdapter.class);
	protected TciTLProvidedV321TT logging;

	private Hashtable<String, LocalAdapterBase> adapterCollection = new Hashtable<String, LocalAdapterBase>();
	
	private TciCDRequired typeServer;

//	private org.etsi.ttcn.tci.Type charstringType;
//	private org.etsi.ttcn.tci.Type integerType;
//  private org.etsi.ttcn.tci.Type floatType;
//  private org.etsi.ttcn.tci.Type booleanType;

	private static TciTMProvided management;

	private String portNameForCoding = "";
	
	private static String ROOTFOLDERTRACEFILE ="./tracefiles/";
	private String testcaseName;	

    private String sMTCName="";
    private String sMTCportName="";
    
	public VxlteAdapter() {
		super();
		PropertyConfigurator.configure("log4j.properties");
		//this.logging = RB.getTciTLProvidedV321TT();
		
		loginfo("VxLTE  Adapter instance created.");
	}
	
	private void loginfo(String message){
		//logging.tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, message);	
		log.info(message);
	}
	
	public TestAdapter setRB(RB rb) {
		super.setRB(rb);
		typeServer = RB.getTciCDRequired();
//		charstringType = typeServer.getCharstring();
//		integerType = typeServer.getInteger();
//		floatType = typeServer.getFloat();
//      booleanType = typeServer.getBoolean();
		management = RB.getTciTMProvided();	
		
		TciCDProvided codec;
		codec=new IOTCodec(RB);
		codecs.put("IOTCodec", codec);
		
		codec=new DIAMETERCodec(RB);
		codecs.put("DIAMETERCodec", codec);
		
		codec=new LibUpperTesterCodec();
		codecs.put("LibUpperTesterCodec", codec);
		
		codec=new SIPCodec(RB);
		((SIPCodec)codec).setLogger(log);
		codecs.put("SIPCodec", codec);
		codecs.put("LibSip V3", codec);
		
		return this;
	}
	
	public  TriStatus triMap(TriPortId comPortId,	TriPortId tsiPortId) {
		
		loginfo("triMap: comPortId=" + comPortId.getComponent()+"-"+ comPortId.getPortName() + "; tsiPortId=" + tsiPortId.getComponent() +"-" +tsiPortId.getPortName());
		
		//CsaDef.triMap(comPortId, tsiPortId);
        
		if (comPortId.getComponent().getComponentName().contains("MTC")){
        	sMTCName=comPortId.getComponent().getComponentName();
        	sMTCportName = tsiPortId.getPortName();
        }

		String portName = comPortId.getComponent().getComponentName()+tsiPortId.getPortName();
		LocalAdapterBase baseAdapter;// = adapterCollection.get(portName);
		
        if (portName.contains("acPort")){
		    baseAdapter = adapterCollection.get(sMTCName/*componentId.getComponentName()+portName*/+tsiPortId.getPortName());
		    
		    if (baseAdapter != null) {
		   	    // baseAdapter.ComponentCollection.put(comPortId.getComponent().getComponentName(), tsiPortId);
		   	    baseAdapter.triMap(comPortId, tsiPortId);	 
		    }else{//new iot connection ... after check of other criteria - new test started
		    }
		}else 
			if (portName.contains("sipPort")){
				baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
				if (baseAdapter != null) {
				  	// baseAdapter.ComponentCollection.put(comPortId.getComponent().getComponentName(), tsiPortId);
				  	baseAdapter.triMap(comPortId, tsiPortId); 	 
				}else{
				}  
		}else 
			if (portName.contains("diameterPort")){
				baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
				if (baseAdapter != null) {
				  	// baseAdapter.ComponentCollection.put(comPortId.getComponent().getComponentName(), tsiPortId);
				  	baseAdapter.triMap(comPortId, tsiPortId);  	 
				}else{
				}  
		}else 
			if (portName.contains("dPort")){
				baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
				if (baseAdapter != null) {
				  	// baseAdapter.ComponentCollection.put(comPortId.getComponent().getComponentName(), tsiPortId);
				  	baseAdapter.triMap(comPortId, tsiPortId); 	 
				}else{
				}  
		}else{
		    baseAdapter = adapterCollection.get(comPortId.getComponent().getComponentName()+portName);
		}
        
		
  		if (baseAdapter != null) {
			return new TriStatusImpl(TriStatus.TRI_OK);
		}
	
		if (portName.contains("acPort")) {
		    //New test started ...
//			try{	
//				PacketCapture pcap=new PacketCapture();
//			}catch(Exception e){
//				loginfo("\nCheck presence of PCAP library on operating system!\n");
//				return new TriStatusImpl(TriStatus.TRI_ERROR);
//			}
			IotConnectionOffline adapter = new IotConnectionOffline(log, Cte);
		    adapter.setTraceFile(ROOTFOLDERTRACEFILE+this.testcaseName+".pcapng");
		    baseAdapter = adapter;
		}else 
		if (portName.contains("eaPort")){
			EaPortConnection adapter = new EaPortConnection(/*this.log,*/ Cte);
		    baseAdapter = adapter;
		}else {
		    loginfo("Port not supported (" + portName +")!\n");
            //Init port connection to avoid mapping errors
		    DummyConnection adapter = new DummyConnection(/*this.log,*/ Cte);
			baseAdapter = adapter;
	    }
		
		adapterCollection.put(portName, baseAdapter);
		return baseAdapter.triMap(comPortId, tsiPortId);
	}

	public TriStatus triUnmap(TriPortId comPortId, TriPortId tsiPortId) {
		
		loginfo("triUnmap: comPortId=" + comPortId.getPortName() + "; tsiPortId="+ tsiPortId.getPortName());
		
		//CsaDef.triUnmap(comPortId, tsiPortId);
		
		String portName = comPortId.getComponent()+tsiPortId.getPortName();
		LocalAdapterBase baseAdapter = adapterCollection.get(portName);
		
		if (portName.contains("acPort")){
		     baseAdapter = adapterCollection.get(sMTCName/*componentId.getComponentName()+portName*/+tsiPortId.getPortName());
		     if (baseAdapter != null) {
		    	 baseAdapter.triUnmap(comPortId, tsiPortId);
		    	 //ComponentCollection.remove(comPortId.getComponent().getComponentName(), tsiPortId);
		     }
		}else
			if (portName.contains("sipPort")){
			     baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
			     if (baseAdapter != null) {
			    	 baseAdapter.triUnmap(comPortId, tsiPortId);
			    	 //ComponentCollection.remove(comPortId.getComponent().getComponentName(), tsiPortId);
			     }
		}else
			if (portName.contains("diameterPort")){
			     baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
			     if (baseAdapter != null) {
			    	 baseAdapter.triUnmap(comPortId, tsiPortId);
			    	 //ComponentCollection.remove(comPortId.getComponent().getComponentName(), tsiPortId);
			     }
		}else
			if (portName.contains("dPort")){
				 baseAdapter = adapterCollection.get(sMTCName+sMTCportName);
				 if (baseAdapter != null) {
					 baseAdapter.triUnmap(comPortId, tsiPortId);
					 //ComponentCollection.remove(comPortId.getComponent().getComponentName(), tsiPortId);
				 }
		}else{
			 baseAdapter = adapterCollection.get(/*comPortId.getComponent().getComponentName()+*/portName);
//			 if (!adapterCollection.isEmpty()) {
//					Enumeration keys = adapterCollection.keys();
//					do {
//						try{
//						String key = (String) keys.nextElement();
//						loginfo("triUnmap:adapters left:key="+key +"comPId=" + portName);		
//						}catch(Exception e){}
//					} while (keys.hasMoreElements());
//				
//				}
		}
		
		if (baseAdapter == null) {
			return new TriStatusImpl(TriStatus.TRI_ERROR);
		}
		
		adapterCollection.remove(portName);
		return baseAdapter.triUnmap();
	}

	public TriStatus triSAReset() {
		
		loginfo("triSAReset: started");

		if (!adapterCollection.isEmpty()) {
			
			Enumeration keys = adapterCollection.keys();
			
			do {
				try{
				String key = (String) keys.nextElement();
				LocalAdapterBase baseAdapter = adapterCollection.get(key);
				
				baseAdapter.triUnmap();				
				adapterCollection.remove(key);
				}catch(Exception e){}
			
			} while (keys.hasMoreElements());
			adapterCollection.clear();
		} 
		
        return super.triSAReset();
	}

	public TriStatus triSend(TriComponentId componentId, TriPortId tsiPortId,	TriAddress triAddress, TriMessage triMessage) {
		
		loginfo("triSend: componentId=" + componentId.getComponentName() + "; tsiPortId="+ tsiPortId.getPortName());
		
		LocalAdapterBase baseAdapter;
		
		String portName = tsiPortId.getPortName();
		portNameForCoding = portName;
		
		if (componentId.getComponentName().contains("MTC")) sMTCName=componentId.getComponentName();
		
	      
        if (portName.contains("acPort")){
		     baseAdapter = adapterCollection.get(sMTCName/*componentId.getComponentName()*/+portName);
		}else{
			 baseAdapter = adapterCollection.get(componentId.getComponentName()+portName);
		}
		
		if (baseAdapter == null) {
			return new TriStatusImpl(TriStatus.TRI_ERROR);
		}

		return baseAdapter.triSend( componentId, tsiPortId,triAddress, triMessage);
    }
	
	public TriStatus triExternalFunction(TriFunctionId functionId,	TriParameterList parameterList, TriParameter returnValue) {

		return new TriStatusImpl(TriStatus.TRI_OK);
	}

	public TciCDProvided getCodec(String s) {
		TciCDProvided codec = null;
		
		//loginfo(">>> TciCDProvided.getCodec for " + portNameForCoding + " = " + ((s != null) ? s : "(null)"));
		loginfo(">>> TciCDProvided.getCodec " + ((s != null) ? s : "(null)"));
		if (s == null  || s.equals("")) {
			
			loginfo("No codec name for usage!!! Last used port = " + portNameForCoding);			
			if (portNameForCoding.equals("") || portNameForCoding == null) {
				s = "IOTCodec";				
			}
		
			else if (portNameForCoding.startsWith("acPort")){
				s= "IOTCodec";
			}else if (portNameForCoding.startsWith("eaPort")){
				s= "LibUpperTesterCodec";
			}else if (portNameForCoding.startsWith("sipPort")){
				s= "SIPCodec";
			}else if (portNameForCoding.startsWith("diameterPort")){
				s= "DIAMETERCodec";
			}
			codec = super.getCodec(s);
		} else {
			codec = super.getCodec(s);
		}
		
//		if (codec != null) {		
//			return codec;
//		}

		return codec;		
	}

	public TriStatus triExecuteTestcase(TriTestCaseId testcase, TriPortIdList tsiList){
		
		//loginfo("Executing TestCase"+ testcase.getTestCaseName());
	    
		this.testcaseName=testcase.getTestCaseName();
		if(Files.exists(Paths.get(ROOTFOLDERTRACEFILE+this.testcaseName+".pcapng")))
	    	return super.triExecuteTestcase(testcase, tsiList);
		else
			return new TriStatusImpl("Check if correct capture file exists!("+this.testcaseName+".pcapng)");
	}
	
	public TciParameterList getTestCaseParameters(TciModuleId moduleId,	String testCaseId) {
		
    	//loginfo("getTestCaseParameters: moduleId=" + moduleId + "; testCaseId=" + testCaseId);
    	
		TciParameterListImpl tciparameterlistimpl = new TciParameterListImpl();
		return tciparameterlistimpl;
	}
}
