package jpcap;

import jpcap.loader.JpcapLibraryLoaderRegistry;
import jpcap.packet.Packet;

/** This class is used to save the captured packets into a file. */
public class JpcapWriter extends JpcapWriterInstance
{	
  private JpcapWriter(JpcapCaptor jpcapCaptor, String filename)
      throws java.io.IOException {
    if (reserveID() < 0) {
      throw new java.io.IOException("Unable to open a writer: "
          + MAX_NUMBER_OF_INSTANCE + " writers are already opened.");
    }
    
    String ret = nativeOpenDumpFile(filename, jpcapCaptor.ID, ID);

    if (ret != null) { // error
      unreserveID();
      throw new java.io.IOException(ret);
    }
  }
	
	/** Opens a file to save the captured packets.
     * @param jpcap instance of JpcapCaptor that was used to capture (load) packets
     * @param filename filename
     * @throws IOException If the file cannot be opened
     */
	public static JpcapWriter openDumpFile(JpcapCaptor jpcap,String filename) throws java.io.IOException{
		return new JpcapWriter(jpcap,filename);
	}
	
	public void close() {
	  nativeClose(ID);
	  unreserveID();
	}
	
	public void writePacket(Packet packet) {
	  nativeWritePacket(packet, ID);
	}
	
	private native String nativeOpenDumpFile(String filename, int captorId, int writerId);
	
	/** Closes the opened file. */
	private native void nativeClose(int id);
	
	/** Saves a packet into the file.
         * @param packet Packet to be saved
         */
	private native void nativeWritePacket(Packet packet, int writerId);

  static {
    JpcapLibraryLoaderRegistry.getInstance().loadLibrary();
  }
}
